@@ -1,14 +1,23 @@
 import binascii
 import warnings
-from typing import Any, Optional, Sequence
+from typing import Any, Iterator, Optional, Sequence
+
+try:
+    from typing import Protocol
+except ImportError:
+    from typing_extensions import Protocol  # type: ignore
 
 from ..utils.base64 import base64, unbase64
 from .connectiontypes import (
     Connection,
     ConnectionArguments,
+    ConnectionConstructor,
     ConnectionCursor,
+    ConnectionType,
     Edge,
+    EdgeConstructor,
     PageInfo,
+    PageInfoConstructor,
 )
 
 __all__ = [
@@ -21,13 +30,24 @@ __all__ = [
 ]
 
 
+class SizedSliceable(Protocol):
+    def __getitem__(self, index: slice) -> Any:
+        ...
+
+    def __iter__(self) -> Iterator:
+        ...
+
+    def __len__(self) -> int:
+        ...
+
+
 def connection_from_array(
-    data: Sequence,
+    data: SizedSliceable,
     args: ConnectionArguments = None,
-    connection_type: Any = Connection,
-    edge_type: Any = Edge,
-    page_info_type: Any = PageInfo,
-) -> Connection:
+    connection_type: ConnectionConstructor = Connection,
+    edge_type: EdgeConstructor = Edge,
+    page_info_type: PageInfoConstructor = PageInfo,
+) -> ConnectionType:
     """Create a connection object from a sequence of objects.
 
     Note that different from its JavaScript counterpart which expects an array,
@@ -54,10 +74,10 @@ def connection_from_array(
 def connection_from_list(
     data: Sequence,
     args: ConnectionArguments = None,
-    connection_type: Any = Connection,
-    edge_type: Any = Edge,
-    pageinfo_type: Any = PageInfo,
-) -> Connection:
+    connection_type: ConnectionConstructor = Connection,
+    edge_type: EdgeConstructor = Edge,
+    pageinfo_type: PageInfoConstructor = PageInfo,
+) -> ConnectionType:
     """Deprecated alias for connection_from_array.
 
     We're now using the JavaScript terminology in Python as well, since list
@@ -79,15 +99,15 @@ def connection_from_list(
 
 
 def connection_from_array_slice(
-    array_slice: Sequence,
+    array_slice: SizedSliceable,
     args: ConnectionArguments = None,
     slice_start: int = 0,
     array_length: int = None,
     array_slice_length: int = None,
-    connection_type: Any = Connection,
-    edge_type: Any = Edge,
-    page_info_type: Any = PageInfo,
-) -> Connection:
+    connection_type: ConnectionConstructor = Connection,
+    edge_type: EdgeConstructor = Edge,
+    page_info_type: PageInfoConstructor = PageInfo,
+) -> ConnectionType:
     """Create a connection object from a slice of the result set.
 
     Note that different from its JavaScript counterpart which expects an array,
@@ -162,13 +182,13 @@ def connection_from_array_slice(
 def connection_from_list_slice(
     list_slice: Sequence,
     args: ConnectionArguments = None,
-    connection_type: Any = Connection,
-    edge_type: Any = Edge,
-    pageinfo_type: Any = PageInfo,
+    connection_type: ConnectionConstructor = Connection,
+    edge_type: EdgeConstructor = Edge,
+    pageinfo_type: PageInfoConstructor = PageInfo,
     slice_start=0,
     list_length=0,
     list_slice_length=None,
-) -> Connection:
+) -> ConnectionType:
     """Deprecated alias for connection_from_array_slice.
 
     We're now using the JavaScript terminology in Python as well, since list
